-- Package declaration
local _library = _G["github.com/eandre/groupauras/core/library"] or {}
_G["github.com/eandre/groupauras/core/library"] = _library

local builtins = _G.lunar_go_builtins

local _aura = _G["github.com/eandre/groupauras/core/aura"]
local _transfer = _G["github.com/eandre/groupauras/core/library/transfer"]


_library.Library = {}

_library.Library._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_library.Library})
	local obj, err
	
	self.listeners = nil
	if type(self.listeners) == type(tbl.listeners) then self.listeners = tbl.listeners end
	self.auras = nil
	if type(self.auras) == type(tbl.auras) then self.auras = tbl.auras end
	self.enabled = nil
	if type(self.enabled) == type(tbl.enabled) then self.enabled = tbl.enabled end
	return self, nil
end

_library.Library._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.listeners) == type(tbl.listeners) then self.listeners = tbl.listeners end
	if type(self.auras) == type(tbl.auras) then self.auras = tbl.auras end
	if type(self.enabled) == type(tbl.enabled) then self.enabled = tbl.enabled end
	return nil
end

_library.New = function(auras)
	local l = setmetatable({ ["auras"] = auras }, {__index=_library.Library})
	_transfer.RegisterListener(l)
	return l
end

_library.Library.RegisterListener = function(l, ln)
	l.listeners = builtins.append(l.listeners, ln)
end

_library.Library.OnAuraReceived = function(l, new)
	for i, old in builtins.slice_iter(l.auras or {}) do
		if old.ID == new.ID then
			if old.Revision < new.Revision then
				l.auras[i + 1] = new
			end
			return
		end
	end
	l.auras = builtins.append(l.auras, new)
end

_library.Library.Auras = function(l)
	return l.auras
end

