-- Package declaration
local _draw = _G["github.com/eandre/groupauras/pkg/draw"] or {}
_G["github.com/eandre/groupauras/pkg/draw"] = _draw

local builtins = _G.lunar_go_builtins

local _context = _G["github.com/eandre/groupauras/pkg/context"]
local _widget = _G["github.com/eandre/lunar-wow/pkg/widget"]

_draw.PointCfg = {}

_draw.PointCfg._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.PointCfg})
	local obj, err
	
	self.Texture = ""
	if type(self.Texture) == type(tbl.Texture) then self.Texture = tbl.Texture end
	self.Text = ""
	if type(self.Text) == type(tbl.Text) then self.Text = tbl.Text end
	self.VertexColor = nil
	if type(self.VertexColor) == type(tbl.VertexColor) then self.VertexColor = tbl.VertexColor end
	self.SizeYards = 0
	if type(self.SizeYards) == type(tbl.SizeYards) then self.SizeYards = tbl.SizeYards end
	self.SizePixels = 0
	if type(self.SizePixels) == type(tbl.SizePixels) then self.SizePixels = tbl.SizePixels end
	self.RotateDegrees = 0
	if type(self.RotateDegrees) == type(tbl.RotateDegrees) then self.RotateDegrees = tbl.RotateDegrees end
	self.RotateSpeed = 0
	if type(self.RotateSpeed) == type(tbl.RotateSpeed) then self.RotateSpeed = tbl.RotateSpeed end
	self.DrawLayer = ""
	if type(self.DrawLayer) == type(tbl.DrawLayer) then self.DrawLayer = tbl.DrawLayer end
	self.DrawOrder = 0
	if type(self.DrawOrder) == type(tbl.DrawOrder) then self.DrawOrder = tbl.DrawOrder end
	return self, nil
end

_draw.PointCfg._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.Texture) == type(tbl.Texture) then self.Texture = tbl.Texture end
	if type(self.Text) == type(tbl.Text) then self.Text = tbl.Text end
	if type(self.VertexColor) == type(tbl.VertexColor) then self.VertexColor = tbl.VertexColor end
	if type(self.SizeYards) == type(tbl.SizeYards) then self.SizeYards = tbl.SizeYards end
	if type(self.SizePixels) == type(tbl.SizePixels) then self.SizePixels = tbl.SizePixels end
	if type(self.RotateDegrees) == type(tbl.RotateDegrees) then self.RotateDegrees = tbl.RotateDegrees end
	if type(self.RotateSpeed) == type(tbl.RotateSpeed) then self.RotateSpeed = tbl.RotateSpeed end
	if type(self.DrawLayer) == type(tbl.DrawLayer) then self.DrawLayer = tbl.DrawLayer end
	if type(self.DrawOrder) == type(tbl.DrawOrder) then self.DrawOrder = tbl.DrawOrder end
	return nil
end

_draw.Point = {}

_draw.Point._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.Point})
	local obj, err
	

		obj, err = _draw.PointCfg._createFromTable(tbl.cfg)
		if err ~= nil then
			return nil, err
		end
		self.cfg = obj
					

		obj, err = _draw.pointFrame._createFromTable(tbl.frame)
		if err ~= nil then
			return nil, err
		end
		self.frame = obj
					
	return self, nil
end

_draw.Point._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		

		obj, err = _draw.PointCfg._createFromTable(tbl.cfg)
		if err ~= nil then
			return err
		end
		self.cfg = obj
					

		obj, err = _draw.pointFrame._createFromTable(tbl.frame)
		if err ~= nil then
			return err
		end
		self.frame = obj
					
	return nil
end

_draw.NewPoint = function(cfg)
	local ctx = cfg.Ctx
	if ctx == nil then
		ctx = _context.Base
	end
	local p = setmetatable({ ["ctx"] = ctx, ["cfg"] = cfg, ["frame"] = _draw.getPointFrame() }, {__index=_draw.Point})
	p:SetTexture(cfg.Texture, cfg.DrawLayer, cfg.DrawOrder)
	p:SetSize(cfg.SizeYards, cfg.SizePixels)
	p:SetText(cfg.Text)
	do
		local c = cfg.VertexColor
		if builtins.length(c) ~= 0 then
			p:SetVertexColor((c[0 + 1] or 0), (c[1 + 1] or 0), (c[2 + 1] or 0), (c[3 + 1] or 0))
		end
	end
	if cfg.RotateSpeed ~= 0 then
		p:Rotate(cfg.RotateDegrees, cfg.RotateSpeed)
	end
	p:FadeIn()
	_draw.markPointActive(p)
	return p
end

_draw.Point.Free = function(p, skipAnimations)
	p.frame:Free(skipAnimations)
	_draw.markPointInactive(p)
	_draw.freePointFrame(p.frame)
end

_draw.Point.Rotate = function(p, degrees, speed)
	p.frame:Rotate(degrees, speed)
end

_draw.Point.SetTexture = function(p, texture, layer, order)
	if layer == "" then
		layer = _widget.LayerArtwork
	end
	p.frame:SetTexture(texture, layer, order)
end

_draw.Point.SetText = function(p, text)
	p.frame:SetText(text)
end

_draw.Point.SetVertexColor = function(p, r, g, b, a)
	p.frame.texture:SetVertexColor(r, g, b, a)
end

_draw.Point.SetSize = function(p, sizeYards, sizePixels)
	if sizeYards ~= 0 then
		p.frame:SetSize(sizeYards * _draw.pixelsPerYard() * 2)
	else
		p.frame:SetSize((sizePixels))
	end
end

_draw.Point.FadeIn = function(p)
	p.frame:FadeIn()
end

_draw.Point.update = function(p)
	if p.ctx:Cancelled() then
		p:Free(false)
		return
	end
	local x, y, inst = p.cfg.Pos:Pos()
	local dx, dy, show = _draw.displayOffset(x, y, inst)
	if (not show) then
		p.frame.frame:Hide()
		return
	end
	p.frame.frame:Show()
	p.frame:SetPosition(dx, dy)
end

_draw.pointFrame = {}

_draw.pointFrame._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.pointFrame})
	local obj, err
	
	self.freed = false
	if type(self.freed) == type(tbl.freed) then self.freed = tbl.freed end

		obj, err = _draw.textureDef._createFromTable(tbl.texDef)
		if err ~= nil then
			return nil, err
		end
		self.texDef = obj
					
	return self, nil
end

_draw.pointFrame._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.freed) == type(tbl.freed) then self.freed = tbl.freed end

		obj, err = _draw.textureDef._createFromTable(tbl.texDef)
		if err ~= nil then
			return err
		end
		self.texDef = obj
					
	return nil
end

_draw.pointFrame.SetTexture = function(f, texture, layer, order)
	local entry = (_draw.textureMap[texture] or nil)
	if entry ~= nil then
		f.texture:SetTexture(entry.Texture)
		f.texture:SetTexCoord(unpack(entry.TexCoords))
		f.texture:SetBlendMode(entry.Blend)
	else
		f.texture:SetTexture(texture)
		f.texture:SetTexCoord(0, 1, 0, 1)
		f.texture:SetBlendMode(_widget.BlendBlend)
	end
	f.texture:SetDrawLayer(layer, order)
	f.texDef = entry
end

_draw.pointFrame.SetText = function(f, text)
	f.text:SetText(text)
end

_draw.pointFrame.SetSize = function(f, pixels)
	if f.texDef ~= nil then
		pixels = pixels * f.texDef.SizeScalar
	end
	f.frame:SetSize(pixels, pixels)
end

_draw.pointFrame.SetPosition = function(f, dx, dy)
	f.frame:ClearAllPoints()
	f.frame:SetPoint("CENTER", f.frame:GetParent(), "CENTER", dx, dy)
end

_draw.pointFrame.Rotate = function(f, degrees, speed)
	local norm = 360 / degrees
	speed = speed * norm
	degrees = (-360)
	if speed < 0 then
		speed = speed * (-1)
		degrees = 360
	end
	f.rotate:SetDuration(speed)
	f.rotate:SetDegrees(degrees)
	f.repeatAnimations:Play()
end

_draw.pointFrame.Reset = function(f)
	f.frame:StopAnimating()
	f.frame:Show()
	f.frame:SetAlpha(1)
	f.repeatAnimations:Stop()
	f.texture:SetVertexColor(1, 1, 1, 1)
	f.text:Show()
	f.texDef = nil
end

_draw.pointFrame.Free = function(f, skipAnimations)
	if skipAnimations then
		f.frame:Hide()
		f.frame:StopAnimating()
		f.text:Hide()
	else
		f:FadeOut()
	end
end

_draw.pointFrame.FadeIn = function(f)
	f.frame:Show()
	f.fadeInAnimations:Play()
end

_draw.pointFrame.FadeOut = function(f)
	f.fadeOutAnimations:Play()
	f.text:Hide()
end

_draw.newPointFrame = function()
	local f = setmetatable({ ["freed"] = false }, {__index=_draw.pointFrame})
	f.frame = _widget.CreateFrame(_draw.canvas)
	f.frame:SetFrameStrata(_widget.StrataMedium)
	f.text = _widget.UIParent():CreateFontString()
	f.text:SetFontObject("GameFontNormal")
	f.text:SetPoint("CENTER", f.frame, "CENTER", 0, 10)
	f.text:SetTextColor(1, 1, 1, 1)
	f.texture = f.frame:CreateTexture()
	f.texture:SetAllPoints(f.frame)
	f.texture:SetDrawLayer(_widget.LayerArtwork, 0)
	f.repeatAnimations = f.frame:CreateAnimationGroup()
	f.repeatAnimations:SetLooping(_widget.LoopRepeat)
	f.rotate = f.repeatAnimations:CreateAnimation(_widget.AnimationRotation)
	f.fadeInAnimations = f.frame:CreateAnimationGroup()
	f.scaleOut = f.fadeInAnimations:CreateAnimation(_widget.AnimationScale)
	f.scaleOut:SetScale(1.5, 1.5)
	f.scaleOut:SetOrder(1)
	local fadeInState = setmetatable({  }, {__index=_draw.animationState})
	f.fadeIn = f.fadeInAnimations:CreateAnimation(_widget.AnimationAlpha)
	f.fadeIn:SetDuration(0.35)
	f.fadeIn:SetScript("OnPlay", function(anim)
		fadeInState:OnLoad(anim)
		f.fadeOutAnimations:Stop()
	end)
	f.fadeIn:SetFromAlpha(0)
	f.fadeIn:SetToAlpha(1)
	f.fadeIn:SetScript("OnUpdate", builtins.create_closure(fadeInState, "Alpha"))
	f.fadeIn:SetScript("OnStop", builtins.create_closure(fadeInState, "FullOpacity"))
	f.fadeIn:SetOrder(2)
	f.scaleIn = f.fadeInAnimations:CreateAnimation(_widget.AnimationScale)
	f.scaleIn:SetDuration(0.35)
	f.scaleIn:SetScale(1 / 1.5, 1 / 1.5)
	f.scaleIn:SetOrder(2)
	local fadeOutState = setmetatable({  }, {__index=_draw.animationState})
	f.fadeOutAnimations = f.frame:CreateAnimationGroup()
	f.fadeOut = f.fadeOutAnimations:CreateAnimation(_widget.AnimationAlpha)
	f.fadeOut:SetFromAlpha(1)
	f.fadeOut:SetToAlpha(0)
	f.fadeOut:SetDuration(0.25)
	f.fadeOut:SetScript("OnFinished", builtins.create_closure(fadeOutState, "HideParent"))
	f.fadeOutAnimations:SetScript("OnPlay", function()
		f.fadeInAnimations:Stop()
	end)
	return f
end

_draw.pointFrameCache = {}

_draw.getPointFrame = function()
	for f in pairs(_draw.pointFrameCache or {}) do
		builtins.delete(_draw.pointFrameCache, f)
		f:Reset()
		f.freed = false
		return f
	end
	return _draw.newPointFrame()
end

_draw.freePointFrame = function(f)
	f.freed = true
	_draw.pointFrameCache[f] = true
end

