-- Package declaration
local _inspect = _G["github.com/eandre/groupauras/pkg/raidutil/inspect"] or {}
_G["github.com/eandre/groupauras/pkg/raidutil/inspect"] = _inspect

local builtins = _G.lunar_go_builtins

local _raidutil = _G["github.com/eandre/groupauras/pkg/raidutil"]
local _luastrings = _G["github.com/eandre/lunar-wow/pkg/luastrings"]
local _widget = _G["github.com/eandre/lunar-wow/pkg/widget"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]
local _lua = _G["github.com/eandre/lunar/lua"]

_inspect.Data = {}

_inspect.Data._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_inspect.Data})
	local obj, err
	
	self.InspectTime = 0
	if type(self.InspectTime) == type(tbl.InspectTime) then self.InspectTime = tbl.InspectTime end
	self.SpecID = 0
	if type(self.SpecID) == type(tbl.SpecID) then self.SpecID = tbl.SpecID end
	return self, nil
end

_inspect.Data._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.InspectTime) == type(tbl.InspectTime) then self.InspectTime = tbl.InspectTime end
	if type(self.SpecID) == type(tbl.SpecID) then self.SpecID = tbl.SpecID end
	return nil
end

_inspect.lastInspectPerGUID = {}

_inspect.lastInspect = 0

_inspect.DataForGUID = function(guid)
	data = (_inspect.lastInspectPerGUID[guid] or nil)
	return data, data ~= nil
end

_inspect.onInspectReady = function(event, args)
	local guid = (args[0 + 1] or nil)
	local now = _wow.GetTime()
	_inspect.lastInspect = now
	do
		local last = (_inspect.lastInspectPerGUID[guid] or nil)
		if last ~= nil and (now - last.InspectTime) < 0.2 then
			return
		end
	end
	local data = { ["InspectTime"] = now, ["SpecID"] = 0}
	_inspect.lastInspectPerGUID[guid] = data
	local unitID, ok = _raidutil.GUIDToUnitID(guid)
	if (not ok) then
		return
	end
	local specID = _wow.GetInspectSpecialization(unitID)
	if specID == 0 then
		return
	end
	data.SpecID = specID
end

builtins.add_init(function()
	_wow.RegisterEvent("INSPECT_READY", _inspect.onInspectReady)
end)

_inspect.inspectQueue = {}

_inspect.nextInspect = function()
	if _wow.IsEncounterInProgress() then
		return
	end
	local inspectFrame = InspectFrame
	if inspectFrame ~= nil and inspectFrame:IsShown() then
		return
	end
	local unqueue = nil
	
	local now = _wow.GetTime()
	for guid, added in pairs(_inspect.inspectQueue or {}) do
		local unitID, ok = _raidutil.GUIDToUnitID(guid)
		if (not ok) then
			unqueue = builtins.append(unqueue, guid)
		elseif _wow.CanInspect(unitID) then
			_wow.NotifyInspect(unitID)
			unqueue = builtins.append(unqueue, guid)
		elseif added < (now - 300) then
			unqueue = builtins.append(unqueue, guid)
		end
	end
	for _, guid in builtins.slice_iter(unqueue or {}) do
		builtins.delete(_inspect.inspectQueue, guid)
	end
end

_inspect.queueGroupInspect = function()
	local prefix = "party"
	local inRaid = _wow.IsInRaid()
	if inRaid then
		prefix = "raid"
	end
	local now = _wow.GetTime()
	local num = _wow.GetNumGroupMembers()
	do
		local i = 1
		while i <= num do
			local online = true
			local unitID = (prefix .. _luastrings.ToString(i))
			if inRaid then
				_, _, _, _, _, _, _, online, _, _, _ = _wow.GetRaidRosterInfo(i)
			else
				if i == num then
					unitID = "player"
				end
			end
			if online then
				local guid = _wow.UnitGUID(unitID)
				_inspect.inspectQueue[guid] = now
			end
			i = i + 1
		end
	end
end

_inspect.timer = (-5)

_inspect.queueTimer = 0

_inspect.timerUpdate = function(dt)
	_inspect.timer = _inspect.timer + dt
	if _inspect.timer >= 3.5 then
		_inspect.queueTimer = _inspect.queueTimer + _inspect.timer
		_inspect.timer = 0
		if _inspect.queueTimer > 60 then
			_inspect.queueTimer = 0
			_inspect.queueGroupInspect()
		end
		_inspect.nextInspect()
	end
end

builtins.add_init(function()
	_wow.RegisterUpdate(_inspect.timerUpdate)
end)

