-- Package declaration
local _rdu = _G["github.com/eandre/groupauras/pkg/raidutil/rdu"] or {}
_G["github.com/eandre/groupauras/pkg/raidutil/rdu"] = _rdu

local builtins = _G.lunar_go_builtins

local _classutil = _G["github.com/eandre/groupauras/pkg/classutil"]
local _context = _G["github.com/eandre/groupauras/pkg/context"]
local _draw = _G["github.com/eandre/groupauras/pkg/draw"]
local _raidutil = _G["github.com/eandre/groupauras/pkg/raidutil"]
local _luastrings = _G["github.com/eandre/lunar-wow/pkg/luastrings"]
local _widget = _G["github.com/eandre/lunar-wow/pkg/widget"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_rdu.ShowGroupMembers = function(ctx)
	_rdu.doShowGroupMembers()
	ctx:OnCancel(nil, function(_, _)
		_rdu.doHideGroupMembers()
	end)
end

_rdu.drawingMember = {}

_rdu.drawingMember._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_rdu.drawingMember})
	local obj, err
	

		obj, err = _draw.Point._createFromTable(tbl.Point)
		if err ~= nil then
			return nil, err
		end
		self.Point = obj
					
	self.GUID = ""
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	self.Counter = 0
	if type(self.Counter) == type(tbl.Counter) then self.Counter = tbl.Counter end
	self.LastMarker = 0
	if type(self.LastMarker) == type(tbl.LastMarker) then self.LastMarker = tbl.LastMarker end
	return self, nil
end

_rdu.drawingMember._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		

		obj, err = _draw.Point._createFromTable(tbl.Point)
		if err ~= nil then
			return err
		end
		self.Point = obj
					
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	if type(self.Counter) == type(tbl.Counter) then self.Counter = tbl.Counter end
	if type(self.LastMarker) == type(tbl.LastMarker) then self.LastMarker = tbl.LastMarker end
	return nil
end

_rdu.currMembers = {}

_rdu.memberCount = 0

_rdu.timer = 0

_rdu.updateTextures = function(dt)
	_rdu.timer = _rdu.timer + dt
	if _rdu.timer < 0.1 then
		return
	end
	_rdu.timer = _rdu.timer - 0.1
	for _, dm in pairs(_rdu.currMembers or {}) do
		local unitID, ok = _raidutil.GUIDToUnitID(dm.GUID)
		if ok then
			local idx = _wow.GetRaidTargetIndex(unitID)
			if idx ~= dm.LastMarker then
				dm.LastMarker = idx
				if idx >= 1 and idx <= 8 then
					dm.Point:SetTexture("marker_" .. _luastrings.ToString(idx), _widget.LayerOverlay, 7)
					dm.Point:SetVertexColor(1, 1, 1, 1)
				else
					local r, g, b = _classutil.Color(unitID)
					dm.Point:SetTexture("party", _widget.LayerOverlay, 7)
					dm.Point:SetVertexColor(r, g, b, 1)
				end
			end
		end
	end
end

_rdu.addMember = function(guid)
	do
		local dm = (_rdu.currMembers[guid] or nil)
		if dm ~= nil then
			dm.Counter = dm.Counter + 1
			return
		end
	end
	local pt = _draw.NewPoint({ ["Pos"] = _draw.GUIDPosition(guid), ["Texture"] = "party", ["VertexColor"] = { 1, 1, 1, 1 }, ["SizeYards"] = 1, ["DrawLayer"] = _widget.LayerOverlay, ["DrawOrder"] = 7, ["Text"] = "", ["SizePixels"] = 0, ["RotateDegrees"] = 0, ["RotateSpeed"] = 0})
	_rdu.currMembers[guid] = { ["Point"] = pt, ["GUID"] = guid, ["Counter"] = 1, ["LastMarker"] = (-1)}
	_rdu.memberCount = _rdu.memberCount + 1
	if _rdu.memberCount == 1 then
		_wow.RegisterUpdate(_rdu.updateTextures)
	end
end

_rdu.removeMember = function(guid)
	local dm = (_rdu.currMembers[guid] or nil)
	if dm ~= nil then
		dm.Counter = dm.Counter - 1
		if dm.Counter <= 0 then
			dm.Point:Free(false)
			builtins.delete(_rdu.currMembers, guid)
			_rdu.memberCount = _rdu.memberCount - 1
		end
	end
	if _rdu.memberCount == 0 then
		_wow.UnregisterUpdate(_rdu.updateTextures)
	end
end

_rdu.ShowUnit = function(ctx, guid)
	_rdu.addMember(guid)
	ctx:OnCancel(nil, function(_, _)
		_rdu.removeMember(guid)
	end)
end

_rdu.doShowGroupMembers = function()
	local inRaid = _wow.IsInRaid()
	local prefix = "party"
	if inRaid then
		prefix = "raid"
	end
	local num = _wow.GetNumGroupMembers()
	do
		local i = 1
		while i <= num do
			local uid = (prefix .. _luastrings.ToString(i))
			if (not inRaid) and i == num then
				uid = "player"
			end
			local guid = _wow.UnitGUID(uid)
			_rdu.addMember(guid)
			i = i + 1
		end
	end
end

_rdu.doHideGroupMembers = function()
	local guids = nil
	
	for _, dm in pairs(_rdu.currMembers or {}) do
		guids = builtins.append(guids, dm.GUID)
	end
	for _, guid in builtins.slice_iter(guids or {}) do
		_rdu.removeMember(guid)
	end
end

